import os,datetime,glob
import multiprocessing
import numpy as np 
import matplotlib as mpl 
mpl.use("Agg") 
from pylab import figure, suptitle, subplot, plot, \
                  xlim, ylim, xticks, yticks, legend, grid, \
                  xlabel, ylabel, title 
import pycdf 
 
ncDir = '/seacoos/data/nccoos/level1/billymitchell/sodar1' 
ncFileGlob = 'billymitchell_sfas_*.nc' 
 
pngDir = '/seacoos/data/nccoos/level1/billymitchell/sodar1/plots/meanspeed' 
pngExt = 'png' 
htmlExt = 'html' 
 
ncFilePattern = os.path.join(ncDir,ncFileGlob) 
files = glob.glob(ncFilePattern)
files.sort()
# files = files[-3:]
previous = [files[-1]] + files[:-1] 
next = files[1:] + [files[0]] 
files = zip(previous,files,next) 
 
html1 = """<html> 
    <head> 
        <title> 
            Billy Mitchell Sodar :: """ 
 
html2 = """ :: Mean Speed 
        </title> 
    </head> 
    <body>
        <form> 
            <a href=\"""" 
 
html3 = """\">&lt;Previous</a>&nbsp;&nbsp;&nbsp; 
            <a href=\"""" 
 
html4 = """\">Next&gt;</a>
        </form>
        <br/> 
        <img src=\"""" 
 
html5 = """\"> 
    </body> 
</html>""" 

def _components(fileSpec):
    "Figure out year and month from file specification."

    fileName = os.path.splitext(os.path.basename(fileSpec))[0]                 
    year = fileName[-7:-3]
    month = fileName[-2:]
    monthName = datetime.datetime(int(year),int(month),1).strftime('%B')
    return (fileName, year, month, monthName)

def meanspeed((previous, ncFile, next), pngDir, genHtml=False, lastMonth=False):
    print 'Processing',ncFile 
    ncFileName, year, month, monthName = _components(ncFile)
    previous, previousYear, previousMonth, previousMonthName = _components(previous)
    next, nextYear, nextMonth, nextMonthName = _components(next)
 
    nc = pycdf.CDF(ncFile) 

    t = nc.var('time')[:] 
    z = nc.var('z')[:]
    u = nc.var('u')[:] 
    v = nc.var('v')[:] 
    w = nc.var('w')[:] 
 
    mu = np.ma.masked_invalid(u) 
    mv = np.ma.masked_invalid(v) 
    mw = np.ma.masked_invalid(w) 
 
    mumean = mu.mean(axis=0)
    mvmean = mv.mean(axis=0)
    mwmean = mw.mean(axis=0)
 
    mustd = mu.std(axis=0) 
    mvstd = mv.std(axis=0) 
    mwstd = mw.std(axis=0) 
 
    ucount = mu.count(axis=0) 
    vcount = mv.count(axis=0) 
    wcount = mw.count(axis=0)

    mrho = np.sqrt((mu**2) + (mv**2))
    mrhomean = mrho.mean(axis=0)
    mrhostd = mrho.std(axis=0)

    mthetamean = (180 * np.arctan2(mv.mean(axis=0), mu.mean(axis=0))/np.pi)
    thetamean = np.piecewise(mthetamean,
                             (mthetamean <= 90, mthetamean > 90),
                             (lambda x: 90 - x, lambda x: 450 - x))

    fig = figure() 
    fig.subplots_adjust(hspace=0.3)
    spt = suptitle('Billy Mitchell Sodar :: ' + monthName + " " + year) 

    subplot(2,4,1) 
    title('Mean Wind Speeds (Cartesian)',fontsize=spt.get_fontsize()*0.5)
    xlabel('Speed (m/s)',fontsize=spt.get_fontsize()*0.6) 
    ylabel('Elevation (m)') 
    xlim(-8,8)
    ylim(0,int(max(z)))
    xticks(fontsize=spt.get_fontsize()*0.6)
    plot(mumean.data,z,label=r'$\bar{u}$') 
    plot(mvmean.data,z,label=r'$\bar{v}$') 
    plot(mwmean.data,z,label=r'$\bar{w}$')
    legend(loc='best',prop={'size':spt.get_fontsize()*0.6})
    grid(True)

    subplot(2,4,2)
    title('Number of Valid Samples',fontsize=spt.get_fontsize()*0.5) 
    xlabel('% of Samples per Month',fontsize=spt.get_fontsize()*0.6)
    xlim(0,100) 
    ylim(0,int(max(z))) 
    xticks(range(0,101,25),fontsize=spt.get_fontsize()*0.6)
    yticks([]) 
    plot((ucount * 100.0)/float(mu.shape[0]),z,label=r'$\%\ u|v$') 
    plot((wcount * 100.0)/float(mw.shape[0]),z,label=r'$\%\ w$') 
    legend(loc='best',prop={'size':spt.get_fontsize()*0.6})
    grid(True)
 
    subplot(2,4,3) 
    title('Mean Wind Velocity (Polar)',fontsize=spt.get_fontsize()*0.5) 
    xlabel('Velocity (m/s)',fontsize=spt.get_fontsize()*0.6)
    xlim(0,10) 
    ylim(0,int(max(z))) 
    xticks(fontsize=spt.get_fontsize()*0.6)
    yticks([]) 
    plot(mrhomean.data,z,label=r'$\bar{\rho}$')
    legend(loc='best',prop={'size':spt.get_fontsize()*0.6})
    grid(True)

    subplot(2,4,4)
    title('Mean Wind Direction (Polar)',fontsize=spt.get_fontsize()*0.5) 
    xlabel('Direction (Windward)',fontsize=spt.get_fontsize()*0.6)
    xlim(0,360)
    ylim(0,int(max(z))) 
    xticks(range(0,361,90),['N','E','S','W','N',],
           fontsize=spt.get_fontsize()*0.6)
    yticks([]) 
    plot(thetamean,z,label=r'$\bar{\theta}$')
    legend(loc='best',prop={'size':spt.get_fontsize()*0.6})
    grid(True)
 
    subplot(2,4,5) 
    title('Wind Speed Std Dev',fontsize=spt.get_fontsize()*0.5)
    xlabel('Std Dev (m/s)',fontsize=spt.get_fontsize()*0.6) 
    ylabel('Elevation (m)') 
    xlim(0,16)
    ylim(0,int(max(z)))
    xticks(fontsize=spt.get_fontsize()*0.6)
    plot(mustd.data,z,label=r'$\sigma_u$') 
    plot(mvstd.data,z,label=r'$\sigma_v$') 
    plot(mwstd.data,z,label=r'$\sigma_w$')
    legend(loc='best',prop={'size':spt.get_fontsize()*0.6})
    grid(True)
 
    subplot(2,4,6)
    title('Number of Valid Samples',fontsize=spt.get_fontsize()*0.5) 
    xlabel('% of Samples per Month',fontsize=spt.get_fontsize()*0.6)
    xlim(0,100) 
    ylim(0,int(max(z))) 
    xticks(range(0,101,25),fontsize=spt.get_fontsize()*0.6)
    yticks([]) 
    plot((ucount * 100.0)/float(mu.shape[0]),z,label=r'$\%\ u|v$') 
    plot((wcount * 100.0)/float(mw.shape[0]),z,label=r'$\%\ w$') 
    legend(loc='best',prop={'size':spt.get_fontsize()*0.6})
    grid(True)

    subplot(2,4,7) 
    title('Wind Velocity Std Dev (Polar)',fontsize=spt.get_fontsize()*0.5) 
    xlabel('Std Dev (m/s)',fontsize=spt.get_fontsize()*0.6)
    xlim(0,10) 
    ylim(0,int(max(z))) 
    xticks(fontsize=spt.get_fontsize()*0.6)
    yticks([]) 
    plot(mrhostd.data,z,label=r'$\sigma_\rho$')
    legend(loc='best',prop={'size':spt.get_fontsize()*0.6})
    grid(True)

    outFile = os.path.join(pngDir,ncFileName + os.extsep + pngExt) 
    print 'Saving', outFile 
    fig.savefig(outFile) 
    fig.clear()
 
    if genHtml:
        htmlFile = os.path.join(pngDir,ncFileName + os.extsep + htmlExt) 
        html = html1 + month + " " + year 
        html = html + html2 + previous + os.extsep + htmlExt
        html = html + html3 + next + os.extsep + htmlExt + html4 
        html = html + os.path.basename(outFile) + html5 
        handle = open(htmlFile,'w') 
        handle.write(html) 
        handle.close() 
 
        if lastMonth: 
            htmlFile = os.path.join(pngDir,"index" + os.extsep + htmlExt) 
            handle = open(htmlFile,'w') 
            handle.write(html) 
            handle.close() 
         
if __name__ == "__main__":
    lastMonth = False
    for previous,ncFile,next in files:              
        if next <= ncFile:
            lastMonth = True
        p = multiprocessing.Process(target=meanspeed,
                                    args=((previous, ncFile, next),
                                           pngDir,
                                           True,
                                           lastMonth))
        p.start()
        p.join()
        if p.exitcode:
           print "Exitcode %s from processing %s" % (p.exitcode, ncFile)
